//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	SasTasks.cc	--- implementation of SasTasks classes
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		UL, 1999-12-06: 1.0.0: - first version
//		UL, 2000-05-03: 1.1.0: - set 'imagebinning' parameter of evselect
//								 explicitely
//
//	AUTHOR:
//		U. Lammers, Mon Dec  6 14:03:43 MET 1999
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/task_devel/packages/srcdisplay/src/SasTasks.cc,v 1.2 2000/11/08 11:46:47 ulammers Exp $
//

#include <fstream>

#include "common.h"
#include "param.h"
#include "ParamInfoSet.h"
#include "SasTasks.h"
#include "attcalc.h"
#include "atthkgen.h"
#include "evselect.h"
#include "srcdisplay.h"
#include "eboxdetect.h"
#include "imgdisplay.h"

Evselect SasTasks::evselect;

void Evselect::image(const string &tableName,
					 const string &imageName,
					 const string &xAxis, const string &yAxis,
					 unsigned binX, unsigned binY)
{
	PROC

	//	bin the event list into an image with evselect
	ParamInfoSet::instance()->resetParamInfo(name());
	openParameters(name());
	setParameter("table", tableName);
	setParameter("keepfilteroutput", false);
	setParameter("withimageset", true);
	setParameter("imageset", imageName);
	setParameter("withimagedatatype", true); // eboxdetect needs int32 images
	setParameter("imagedatatype", "Int32");
	setParameter("xcolumn", xAxis);
	setParameter("ycolumn", yAxis);
	setParameter("imagebinning", "binSize");
	setParameter("ximagebinsize", DBL(binX));
	setParameter("yimagebinsize", DBL(binY));

	evselect();

	ENDPROC
} // Evselect::image()


Eboxdetect SasTasks::eboxdetect;

void Eboxdetect::detect(const string &imageSetName, const string &boxSetName,
					    unsigned nruns, double likemin)
{
	PROC

	strstream s;
	s<<"eboxdetect imagesets="<<imageSetName<<" boxlistset="<<boxSetName
	 <<" boxsize=5 nruns="<<INT(nruns)<<" likemin="<<likemin
	 <<" usematchedfilter=false usemap=false withdetmask=false"
	 <<" withexpimage=false"<<ends;
#if 0
	//	run eboxdetect
	ParamInfoSet::instance()->resetParamInfo(name());
	openParameters(name());
	vector<string> is;
	is.push_back(imageSetName);
	setParameter("imagesets", is);
	setParameter("boxlistset", boxSetName);
	setParameter("boxsize", 5);
	setParameter("nruns", INT(nruns));
	setParameter("likemin", likemin);
	setParameter("usematchedfilter", false);
	setParameter("usemap", false);
	setParameter("withdetmask", false);
	setParameter("withexpimage", false);

	eboxdetect();
#endif
	char *p = s.str();

	errstr<<"Running eboxdetect:\n";
	errstr<<p<<endl<<Sas::message(Msg::Lib, Msg::Sparse);
	system(p);
	delete p;

	//	check if we found any sources
	ifstream f(boxSetName.c_str());
	if (!f)
		errstr<<"eboxdetect found no sources in image `"<<imageSetName
			  <<"' - try to decrease detection likelihood"
			  <<Sas::error("NoSourcesFound");

	ENDPROC
} // Eboxdetect::detect()


Srcdisplay	SasTasks::srcdisplay;

void Srcdisplay::show(const string &imageSetName, const string &boxSetName,
					  double sourceRadius, bool includeSources)
{
	PROC

	ParamInfoSet::instance()->resetParamInfo(name());
	openParameters(name());
	setParameter("boxlistset", boxSetName);
	setParameter("imageset", imageSetName);
	setParameter("sourceradius", sourceRadius);
	setParameter("includesources", includeSources);

	srcdisplay();

	ENDPROC
} // Srcdisplay::show()


Imgdisplay SasTasks::imgdisplay;

vector<string> Imgdisplay::getRegions()
{
	PROC

	//	get regions from imgdisplay
	ParamInfoSet::instance()->resetParamInfo(name());
	openParameters(name());
	setParameter("withimagefile", false);
	setParameter("withregion", true);
	imgdisplay();

	RETURN stringListParameter("regions");
} // Imgdisplay::getRegions()

Attcalc SasTasks::attcalc;

void Attcalc::run(const string &eventListName)
{
	PROC
		
	ParamInfoSet::instance()->resetParamInfo("atthkgen");
	openParameters("atthkgen");
	string atthkset(stringParameter("atthkset"));
	ifstream f(atthkset.c_str());
	if (!f)
		//	does not exist - create it
		atthkgen();

	ParamInfoSet::instance()->resetParamInfo(name());
	openParameters(name());
	setParameter("eventset", eventListName);
	setParameter("attitudelabel", "ahf");
	setParameter("refpointlabel", "pnt");
	setParameter("atthkset", atthkset);
	attcalc();

	ENDPROC
} // Attcalc::run()
