//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	SasTasks.h	--- I/F to Sas tasks
//
//	DESCRIPTION:
//		A task object represents a runnable SAS task.
//
//	CHANGE HISTORY:
//		UL, 1999-12-06: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers, Mon Dec  6 14:03:43 MET 1999
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/task_devel/packages/srcdisplay/src/SasTasks.h,v 1.2 2000/11/08 11:46:47 ulammers Exp $
//

#ifndef TASK_H
#define TASK_H

#include <vector>
#include <string>
#include "Tracer.h"

class SasTask {
public:
	SasTask(const string &name) { PROC _name = name; ENDPROC }
	const string &name() const { PROC RETURN _name; }
	virtual void run() const { PROC errstr<<"Do not know how to run `"
										  <<_name<<"'"
										  <<Sas::error("NotRunnable");
							   ENDPROC }
	virtual ~SasTask() { PROC ENDPROC }

private:
	string	_name;
};

class Evselect : public SasTask {
public:
	Evselect() : SasTask("evselect") { PROC ENDPROC }

	void image(const string &tableName,
			   const string &imageName,
			   const string &xAxis, const string &yAxis,
			   unsigned binX, unsigned binY);
		//
		//	PURPOSE:
		//		Accumulate an image from an event list with evselect
		//
		//	ARGUMENTS:
		//		tableName	: name of event list table
		//		imageName	: name of image set to create
		//		xAxis/yAxis	: column names in event list from which to
		//					  accumulate the image
		//		binX/binY	: bin sizes in X/Y
		//

	virtual ~Evselect() { PROC ENDPROC }
};


class Eboxdetect : public SasTask {
public:
	Eboxdetect() : SasTask("eboxdetect") { PROC ENDPROC }

	void detect(const string &imageSetName, const string &boxSetName,
				unsigned nruns = 1, double likemin = 15.);

	virtual ~Eboxdetect() { PROC ENDPROC }
};



class Srcdisplay : public SasTask {
public:
	Srcdisplay() : SasTask("srcdisplay") { PROC ENDPROC }

	void show(const string &imageSetName, const string &boxSetName,
			  double sourceRadius = .01, bool includeSources = true);

	virtual ~Srcdisplay() { PROC ENDPROC }
};


class Imgdisplay : public SasTask {
public:
	Imgdisplay() : SasTask("imgdisplay") { PROC ENDPROC }

	vector<string> getRegions();

	virtual ~Imgdisplay() { PROC ENDPROC }
};


class Attcalc : public SasTask {
public:
	Attcalc() : SasTask("attcalc") { PROC ENDPROC }

	void run(const string &eventListName);
		//
		//	PURPOSE:
		//		Run attcalc with 'attitudelabel=ahf'/'refpointlabel=pnt'
		//
		//	ARGUMENTS:
		//		eventListName	: name of event list
		//

	virtual ~Attcalc() { PROC ENDPROC }
};


class SasTasks {
public:
	static Evselect		evselect;
	static Eboxdetect	eboxdetect;
	static Srcdisplay	srcdisplay;
	static Imgdisplay	imgdisplay;
	static Attcalc		attcalc;
};

#endif
